@@ -214,9 +214,13 @@ def _update_kwargs(original_kwargs: dict,
 
 
 class ConfiguredAction:
+    # configured action registry
+    _config_registry: List['ConfiguredAction'] = []
+
     def __init__(self,
                  action: Union[Callable, Action, str],
                  *args,
+                 configuration_name: str = None,
                  **kwargs,
                  ):
         """
@@ -224,13 +228,24 @@ class ConfiguredAction:
 
         :param action: action to perform (callable)
         :param args: arguments for the action
+        :param configuration_name: optional name for the configuration (for key retrieval)
         :param kwargs: keyword arguments for the action
         """
         self._action: Action = None
+        self._config_name: str = None
         # register action and store attribute
         self.action = action
         self.args = _update_args(self.action.default_args, args)
         self.kwargs = _update_kwargs(self.action.default_kwargs, kwargs)
+        # generate name if not specified
+        if configuration_name is None:
+            configuration_name = self._generate_unique_name(self.action)
+        else:
+            if configuration_name in self.get_configuration_names():
+                raise NameError(f'The configuration name "{configuration_name}" is already used, please choose a '
+                                f'unique name for the configuration. ')
+        self.name = configuration_name
+        self._config_registry.append(self)
 
     def __str__(self):
         return f'{self.__class__.__name__} {self.name}'
@@ -245,6 +260,7 @@ class ConfiguredAction:
         return self.__class__(
             self.action,
             self.args,
+            self.name,
             self.kwargs,
         )
 
@@ -263,8 +279,17 @@ class ConfiguredAction:
 
     @property
     def name(self) -> str:
-        """method name"""
-        return self.action.name
+        """name for the configuration"""
+        return self._config_name
+
+    @name.setter
+    def name(self, value: str):
+        if value is not None:
+            # check if already defined
+            if value in self.get_configuration_names():
+                raise NameError(f'The configuration name "{value}" is already used, please choose a '
+                                f'unique name for the configuration. ')
+            self._config_name = value
 
     def update_args(self, *args):
         """Updates the arguments for the Action"""
@@ -295,6 +320,62 @@ class ConfiguredAction:
         """status string for the Action"""
         return f'{self.status_code}: {self.status_string}'
 
+    @classmethod
+    def _generate_unique_name(cls, action: Action) -> str:
+        """
+        Generates a unique numbered name based on an Action instance. The resulting name will be unique in the
+        registry.
+
+        :param action: action to base naming off of
+        :return: generated name
+        """
+        i = 1
+        current_names = cls.get_configuration_names()
+        while True:
+            target_name = f'{action.name}_{i}'
+            if f'{action.name}_{i}' not in current_names:
+                return target_name
+            i += 1
+
+    @classmethod
+    def get_configuration_by_name(cls, configuration_name: str) -> 'ConfiguredAction':
+        """
+        Retrieves a ConfiguredAction by name
+
+        :param configuration_name: name identifier of configuration
+        :return: ConfiguredAction
+        """
+        for config in cls._config_registry:
+            if config.name == configuration_name:
+                return config
+        raise NameError(f'The key "{configuration_name}" does not match any ConfiguredAction instances')
+
+    @classmethod
+    def get_configuration_names(cls) -> List[str]:
+        """
+        Retrieves the names of all configurations.
+
+        Warning: return will be sorted, but order does not correspond to order in registry (indexing based on this
+        return is not recommended).
+        """
+        return sorted([config.name for config in cls._config_registry])
+
+    @classmethod
+    def get_configurations_of_action(cls, action: Union[str, Action]) -> List['ConfiguredAction']:
+        """
+        Retrieves all of the Configurations associated with the specified action
+
+        :param action: action to reference
+        :return: list of ConfiguredAction instances
+        """
+        if isinstance(action, Action) is False:
+            action = Action.register_action(action)
+        return [
+            config
+            for config in cls._config_registry
+            if config.action is action
+        ]
+
     def as_dict(self) -> dict:
         """dictionary of relevant information"""
         out = {
