@@ -44,4 +44,4 @@ step_b.next_step = if_check
 automation = Automation(first_step=step_a,
                         **automation_kwargs)
 
-automation()
\ No newline at end of file
+# automation()
\ No newline at end of file
